<?php
/* SVN FILE: $Id: dbo.php 1114 2005-10-09 01:56:21Z phpnut $ */
/**
 *
 * PHP versions 4
 *
 * CakePHP :  Rapid Development Framework <http://www.cakephp.org/>
 * Copyright (c) 2005, CakePHP Authors/Developers
 *
 * Author(s): Michal Tatarynowicz aka Pies <tatarynowicz@gmail.com>
 *            Larry E. Masters aka PhpNut <nut@phpnut.com>
 *            Kamil Dzielinski aka Brego <brego.dk@gmail.com>
 *
 * Restructed by D.Sheiko
 *
 * @filesource 
 * @author       CakePHP Authors/Developers
 * @copyright    Copyright (c) 2005, CakePHP Authors/Developers
 * @link         https://trac.cakephp.org/wiki/Authors Authors/Developers
 * @package      cake
 * @subpackage   cake.cake.libs.model.dbo
 * @since        CakePHP v 0.2.9
 * @version      $Revision: 1114 $
 * @modifiedby   $LastChangedBy: phpnut $
 * @lastmodified $Date: 2005-10-08 20:56:21 -0500 (Sat, 08 Oct 2005) $
 * @license      http://www.opensource.org/licenses/mit-license.php The MIT License
 */
// $Id: dbo.php,v 1.0 2005/11/10 16:39:06 korvin Exp $


/**
 * Purpose: DBO/ADO
 * 
 * Description: 
 * A SQL functions wrapper. Provides ability to get results as arrays
 * and query logging (with execution time).
 *
 * Example usage:
 *
 * <code>
 * require_once('dbo_mysql.php'); // or 'dbo_postgres.php'
 *
 * // create and connect the object
 * $db = new DBO_MySQL(array( // or 'DBO_Postgres'
 *    'host'=>'localhost',
 *    'login'=>'username',
 *    'password'=>'password',
 *    'database'=>'database'));
 *
 *  // read the whole query result array (of rows)
 * $all_rows = $db->all("SELECT a,b,c FROM table");
 *
 *  // read the first row with debugging on
 *  $first_row_only = $db->one("SELECT a,b,c FROM table WHERE a=1", TRUE);
 *
 *  // emulate the usual way of reading query results
 *  if ($db->query("SELECT a,b,c FROM table")) 
 *	{
 *  	while ( $row = $db->farr() ) 
 *	  	{
 *      	print $row['a'].$row['b'].$row['c'];
 *  	}
 *  }
 *
 * // show a log of all queries, sorted by execution time
 * $db->showLog(TRUE);
 * </code>
 *
 * @package    cake
 * @subpackage cake.cake.libs.model.dbo
 * @since      CakePHP v 0.2.9
 */

class DBO
{

	/**
	 * Are we connected to the database?
	 *
	 * @var boolean
	 * @access public
	 */
	var $connected=FALSE;

	/**
	 * Connection configuration.
	 *
	 * @var array
	 * @access public
	 */
	var $config=FALSE;

	/**
	 * Enter description here...
	 *
	 * @var boolean
	 * @access public
	 */
	var $debug=FALSE;

	/**
	 * Enter description here...
	 *
	 * @var boolean
	 * @access public
	 */
	var $fullDebug=FALSE;

	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 * @access public
	 */
	var $error= false;

	/**
	 * String to hold how many rows were affected by the last SQL operation.
	 *
	 * @var unknown_type
	 * @access public
	 */
	var $affected= false;

	/**
	 * Number of rows in current resultset
	 *
	 * @var int
	 * @access public
	 */
	var $numRows= false;

	/**
	 * Time the last query took
	 *
	 * @var unknown_type
	 * @access public
	 */
	var $took= false;

	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 * @access private
	 */
	var $_conn= false;

	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 * @access private
	 */
	var $_result= false;

	/**
	 * Queries count.
	 *
	 * @var unknown_type
	 * @access private
	 */
	var $_queriesCnt=0;

	/**
	 * Total duration of all queries.
	 *
	 * @var unknown_type
	 * @access private
	 */
	var $_queriesTime= false;

	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 * @access private
	 */
	var $_queriesLog=array();

	/**
	 * Maximum number of items in query log, to prevent query log taking over
	 * too much memory on large amounts of queries -- I we've had problems at 
	 * >6000 queries on one system.
	 *
	 * @var int Maximum number of queries in the queries log.
	 * @access private
	 */
	var $_queriesLogMax=200;


	var $limit_finish=0;
	var $limit_start=0;
	var $where = "";
	var $allchannels = false;
	var $length = 0;
	var $order = false;
	var $range = false;	
	/**
	 * Constructor. Sets the level of debug for dbo (fullDebug or debug). 
	 *
	 * @param array $config
	 * @return unknown
	 */
	function DBO($config= false) {
		$this->debug = DEBUG > 0;
		$this->fullDebug = DEBUG > 1;
		return $this->connect($config);
	}
	/**
	 * Destructor. Closes connection to the database.
	 *
	 */
	function destroy() {
		$this->close();
	}
	/**
	 * Returns a string with a USE [databasename] SQL statement.
	 *
	 * @param string $db_name Name of database to use
	 * @return unknown Result of the query
	 */
	function useDb($db_name) {
		return $this->query("USE {$db_name}");
	}
	/**
	 * Disconnects database, kills the connection and says the connection is closed, and if DEBUG is turned on, the log for this object is shown.
	 *
	 */
	function close () {
		if ($this->fullDebug) $this->showLog();
		$this->disconnect();
		$this->_conn =  false;
		$this->connected = false;
	}

	/**
	 * Prepares a value, or an array of values for database queries by quoting and escaping them.
	 *
	 * @param mixed $data A value or an array of values to prepare.
	 * @return mixed Prepared value or array of values.
	 */
	function prepare ($data) {
		if (is_array($data)) {
			$out =  false;
			foreach ($data as $key=>$item) {
				$out[$key] = $this->prepareValue($item);
			}
			return $out;
		}
		else {
			return $this->prepareValue($data);
		}
	}

	function tables() {
		return array_map('strtolower', $this->tablesList());
	}

	/**
	 * Executes given SQL statement.
	 *
	 * @param string $sql SQL statement
	 * @return unknown
	 */
	function rawQuery ($sql) {
		$this->took = $this->error = $this->numRows = false;
		return $this->execute($sql);
	}

	/**
	 * Queries the database with given SQL statement, and obtains some metadata about the result 
	 * (rows affected, timing, any errors, number of rows in resultset). The query is also logged. 
	 * If DEBUG is set, the log is shown all the time, else it is only shown on errors.
	 *
	 * @param string $sql
	 * @return unknown
	 */
	function query($sql) {
		$t = getMicrotime();
		$this->_result = $this->execute($sql);
		$this->affected = $this->lastAffected();
		$this->took = round((getMicrotime()-$t)*1000, 0);
		$this->error = $this->lastError();
		$this->numRows = $this->lastNumRows($this->_result);
		$this->logQuery($sql);
		
		if ($this->debug && $this->error)
			message_die("<div style=\"background-color: white; padding: 5px; margin: 5px 0px 5px 0px; color: black;\">".($sql)."</div><b>SQL Error:</b> ".convert_cyr_string($this->error,'k','w'));
		
		return $this->error? false: $this->_result;
	}

	/**
	 * Returns a single row of results from the _last_ SQL query.
	 *
	 * @param resource $res
	 * @return array A single row of results
	 */
	function farr ($assoc=false) {
		if ($assoc === false) {
			return $this->FetchRow();
		}
		else {
			return $this->FetchRow($assoc);
		}
	}

	/**
	 * Returns a single row of results for a _given_ SQL query.
	 *
	 * @param string $sql SQL statement
	 * @return array A single row of results
	 */
	function one ($sql) {
		return $this->query($sql)? $this->farr(): false;
	}

	/**
	 * Returns an array of all result rows for a given SQL query. 
	 * Returns false if no rows matched.
	 *
	 * @param string $sql SQL statement
	 * @return array Array of resultset rows, or false if no rows matched
	 */
	function all ($sql)
	{
		if($this->query($sql))
		{
			$out=array();
			while ($item = $this->farr( false, true))
			{
				$out[] = $item;
			}
			return $out;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Returns a single field of the first of query results for a given SQL query, or false if empty.
	 *
	 * @param string $name Name of the field
	 * @param string $sql SQL query
	 * @return unknown
	 */
	function field ($name, $sql) {
		$data = $this->one($sql);
		return empty($data[$name])? false: $data[$name];
	}

	/**
	 * Checks if the specified table contains any record matching specified SQL
	 *
	 * @param string $table Name of table to look in
	 * @param string $sql SQL WHERE clause (condition only, not the "WHERE" part)
	 * @return boolean True if the table has a matching record, else false
	 */
	function hasAny($table, $sql) {
		$out = $this->one("SELECT COUNT(*) AS count FROM {$table}".($sql? " WHERE {$sql}":""));
		return is_array($out)? $out[0]['count']: false;
	}
	/**
	 * Checks if it's connected to the database
	 *
	 * @return boolean True if the database is connected, else false
	 */
	function isConnected() {
		return $this->connected;
	}
	/**
	 * Outputs the contents of the log.
	 *
	 * @param boolean $sorted
	 */
	function showLog($sorted=false) {
		$log = $sorted?
		sortByKey($this->_queriesLog, 'took', 'desc', SORT_NUMERIC):
		$this->_queriesLog;
		print("<table border=1>\n<tr><th colspan=7>{$this->_queriesCnt} queries took {$this->_queriesTime} ms</th></tr>\n");
		print("<tr><td>Nr</td><td>Query</td><td>Error</td><td>Affected</td><td>Num. rows</td><td>Took (ms)</td></tr>\n");
		foreach($log AS $k=>$i) {
			print("<tr><td>".($k+1)."</td><td>{$i['query']}</td><td>{$i['error']}</td><td align='right'>{$i['affected']}</td><td align='right'>{$i['numRows']}</td><td align='right'>{$i['took']}</td></tr>\n");
		}
		print("</table>\n");
	}

	/**
	 * Log given SQL query.
	 *
	 * @param string $sql SQL statement
	 */
	function logQuery($sql) {
		$this->_queriesCnt++;
		$this->_queriesTime += $this->took;
		$this->_queriesLog[] = array(
		'query'=>$sql,
		'error'=>$this->error,
		'affected'=>$this->affected,
		'numRows'=>$this->numRows,
		'took'=>$this->took
		);
		if (count($this->_queriesLog) > $this->_queriesLogMax) {
			array_pop($this->_queriesLog);
		}
		if ($this->error) return false; // shouldn't we be logging errors somehow?
	}

	/**
	 * Output information about an SQL query. The SQL statement, number of rows in resultset, 
	 * and execution time in microseconds. If the query fails, and error is output instead.
	 *
	 * @param string $sql
	 */
	 /*
	function showQuery($sql) {
		$error = $this->error;
		if (strlen($sql)>200 && !$this->fullDebug) {
			$sql = substr($sql, 0, 200) .'[...]';
		}
		if ($this->debug || $error) {
			print("<p style=\"text-align:left\"><b>Query:</b> {$sql} <small>[Aff:{$this->affected} Num:{$this->numRows} Took:{$this->took}ms]</small>");
			if($error) {
				print("<br /><span style=\"color:Red;text-align:left\"><b>ERROR:</b> {$this->error}</span>");
			}
			print('</p>');
		}
	}
	*/
	
	
	/**
	 * Clear results in array
	 *
	 * @param array 
	 * @return array 
	 * @access public
	 */	
	function clearArray($result) {
		if($result) {foreach($result as $key=>$val){ if(is_int($key)) unset($result[$key]); }}
		return $result;
	}
	/**
	 * Strip slashes in array
	 *
	 * @param array 
	 * @return array 
	 * @access public
	 */	
	function stripSlahes($result) {
		$key = 0; $val =0;
			if( get_magic_quotes_gpc() ) {
				if($result) {foreach($result as $key=>$val){
					$result[$key]=stripslashes($result[$key]);}
				}
			}
		return $result;
	}	
	
	/**
	 * Get one record cleared results of a SQL query
	 *
	 * @param string SQL commands
	 * @return array SQL results array
	 * @access public
	 */	
	function row_fetch_array($query) { 
		$result=$this->one($query);
		$result = $this->clearArray($result);
		$result = $this->stripSlahes($result);
		return $result;
	} 	
		
	/**
	 * Get a SQL
	 *
	 * @param string SQL commands
	 * @return array SQL results array
	 * @access public
	 */	
	function get_sql($sql) {
		$objdata = array();
		$fetch = array();
		$f = array();
		$r = false;
		$r1 = false;
		$r=$this->query($sql);
		while($fetch = $r->fetchRow())	{ 
			$fetch = $this->clearArray($fetch);
			$fetch = $this->stripSlahes($fetch);
			$sql = "SELECT B.var, B.data FROM ".DBPREFIX."data B WHERE B.parent_id = ".$fetch["id"];
			$data = $this->all($sql);
			if($data)
				foreach($data as $datafetch) {
					if($datafetch) {
					$datafetch = $this->stripSlahes($datafetch);
					$objdata[ $fetch["id"] ][$datafetch["var"]]=$datafetch["data"];
					}
				}
				unset($data);
				$objdata[ $fetch["id"] ]["cdate"] = $fetch["cdate"];
				$objdata[ $fetch["id"] ]["mdate"] = $fetch["mdate"];
				$objdata[ $fetch["id"] ]["ip"] = $fetch["ip"];
				$objdata[ $fetch["id"] ]["parent_id"] = $fetch["parent_id"];
				$objdata[ $fetch["id"] ]["ID"] = $objdata[ $fetch["id"] ]["id"] = $fetch["id"];
			}
		return $objdata;				
	}
	

	/**
	 * Get data of specified parent or data of specified id
	 *
	 * @param integer $parent_id 
	 * @param integer $id	 
	 * @return array Records array
	 * @access public
	 */	
	function get_data($parent_id=false, $id = false){
		global $sapi_obj;
		
		$objdata = array();
		$limit = "";
		$sql = "";
		$fetch = array();
		$f = array();
		$r = false;
		$r1 = false;
		
		$order = " ORDER by A.cdate DESC";
		if($this->order=="datecasc") $order = " ORDER by A.cdate ASC";
		if($this->order=="datecdesc") $order = " ORDER by A.cdate DESC";
		if($this->order=="index_asc" AND $sapi_obj->env["index"]) { $order = " ORDER by B.data ASC"; $this->where = " AND B.var='".$sapi_obj->env["index"]."' "; }
		if($this->order=="index_desc" AND $sapi_obj->env["index"]) { $order = " ORDER by B.data DESC"; $this->where = " AND B.var='".$sapi_obj->env["index"]."' "; }
		if($this->limit_finish) $limit = " LIMIT ".(int)$this->limit_start.", ".($this->limit_finish-$this->limit_start+1);
		
		if($this->range!="anywhere")
		$where_channel = "AND A.parent_id=".$parent_id."";
		
		if($this->allchannels) $where_channel = "";
		
		// Number of rows
		if($id) 
			$this->length = 1;
		else {
			$sql = "SELECT A.id, UNIX_TIMESTAMP(A.cdate) as cdate,  UNIX_TIMESTAMP(A.mdate) as mdate, A.ip, 
			B.var, B.data FROM ".DBPREFIX."tree A, ".DBPREFIX."data B WHERE A.id=B.parent_id $where_channel ".$this->where." GROUP by A.id";
			
			$r=$this->query($sql);
			$this->length = $r->_numOfRows;
		}
		
		// Get data
		if($id) 
			$sql = "SELECT A.id, A.parent_id, UNIX_TIMESTAMP(A.cdate) as cdate,  UNIX_TIMESTAMP(A.mdate) as mdate, A.ip, 
			B.var, B.data FROM ".DBPREFIX."tree A, ".DBPREFIX."data B WHERE A.id=B.parent_id AND A.id=".$id;
		else 		
			$sql = "SELECT A.id, A.parent_id, UNIX_TIMESTAMP(A.cdate) as cdate,  UNIX_TIMESTAMP(A.mdate) as mdate, A.ip FROM ".DBPREFIX."tree A, ".DBPREFIX."data B WHERE A.id=B.parent_id $where_channel ".$this->where." GROUP by A.id ".$order.$limit;

		$r=$this->query($sql);
		$this->where = "";
		while($fetch = $r->fetchRow($r))	{ 
			if( $fetch ) {
				$fetch = $this->clearArray($fetch);
				$fetch = $this->stripSlahes($fetch);
				$sql = "SELECT B.var, B.data FROM ".DBPREFIX."data B WHERE B.parent_id = ".$fetch["id"];
				$data = $this->all($sql);
				if($data) {
					foreach($data as $datafetch) {
						if($datafetch) {
						$datafetch = $this->stripSlahes($datafetch);
						$objdata[ $fetch["id"] ][strtolower($datafetch["var"])]=
						$objdata[ $fetch["id"] ][strtoupper($datafetch["var"])]=$datafetch["data"];
						}
					}
					unset($data);				
				}
				$objdata[ $fetch["id"] ]["cdate"] = $fetch["cdate"];
				$objdata[ $fetch["id"] ]["mdate"] = $fetch["mdate"];
				$objdata[ $fetch["id"] ]["ip"] = $fetch["ip"];
				$objdata[ $fetch["id"] ]["ID"] = $objdata[ $fetch["id"] ]["id"] = $fetch["id"];
				$objdata[ $fetch["id"] ]["parent_id"] = $fetch["parent_id"];
			}
		} 
		return $objdata;
	}
	

	/**
	 * Get data of specified QC pointer
	 *
	 * @param integer document id	 
	 * @param string QC name 	 
	 * @return array Data
	 * @access public
	 */	
	function get_qcdata($id = false, $qc=false){
		global $sapi_obj;
		$sql = "";
		$fetch = array();
		$f = array();
		$r = false;
		$sql = "SELECT A.data FROM ".DBPREFIX."data A WHERE A.parent_id=".$id." AND A.var='".$qc."' LIMIT 0,1";
		$fetch = $this->one($sql);
		return $fetch["data"];		
	}
	
		
	/**
	 * Get navigation indexes of current infochannel
	 *
	 * @param integer parent id	 
	 * @param integer document id	 
	 * @param string order command 	 
	 * @return array list of prev and next ids
	 * @access public
	 */	
	function get_navindexes($parent_id, $id, $ORDER){
		global $sapi_obj;
		
		$objdata = array();
		$sql = "";
		$fetch = array();
		$prev_id = false;
		$next_id = false;
		
		$order = " ORDER by A.cdate ";
		
		// Current 
		$sql = "SELECT A.cdate FROM ".DBPREFIX."tree A WHERE A.parent_id=".$parent_id." AND A.id=".$id." LIMIT 0,1";
		$cdate = $this->one($sql);
		
		// Previous
		$sql = "SELECT A.id FROM ".DBPREFIX."tree A WHERE A.parent_id=".$parent_id." AND A.cdate>'".$cdate["cdate"]."' $order ASC LIMIT 0,1";
		$fetch = $this->one($sql);
		$prev_id = $fetch["id"];
		
			
		// Next
		$sql = "SELECT A.id FROM ".DBPREFIX."tree A WHERE A.parent_id=".$parent_id." AND A.cdate<'".$cdate["cdate"]."' $order DESC LIMIT 0,1";
		$fetch = $this->one($sql);
		$next_id = $fetch["id"];
		
		return array($prev_id, $next_id);
	}	
	
	/**
	 * Save data of record
	 *
	 * Appropriate values of $data["post_action"]:
	 * ADDNEWIPROW
	 * ADDROW
	 * CHANGEROW
	 * UPDATEROW
	 * Source array $data["ROW"][..]
	 *
	 * @param integer parent id	 
	 * @param array data array
	 * @param string unique IP for filtering 
	 * @return integer new ID
	 * @access public
	 */	
	function save_data_row($parent_id=false, $data=false, $unique_ip=false) {
		
		$IP = getenv("REMOTE_ADDR");
		$DATE_CREATE = date("Y-m-d H:i:s");
		$r = false;
		$flag_ip_false = false;
		$sql = "";
		$fetch = array();
		$index = "";
		$val = "";
		$lastid = 0;
		
		if (!$data) $data=$_POST;
		if($data["ROW"]) $data["row"] = $data["ROW"];
		if($data["row"]) {
			
			if(!$data["row"]["IP"]) $data["row"]["IP"] = $IP; else $IP = $data["row"]["IP"];
			if(!$data["row"]["DATE_CREATE"]) $data["row"]["DATE_CREATE"] = $DATE_CREATE; else $DATE_CREATE = $data["row"]["DATE_CREATE"];
			
			$sql = "SELECT id FROM ".DBPREFIX."tree WHERE ip='".$IP."'";
			$r = $this->query($sql); 
			$flag_ip_false = $r->_numOfRows;
			
			if((!$flag_ip_false AND $data["post_action"]=="ADDNEWIPROW" ) OR $data["post_action"] =="ADDROW" ) {
				// Adding new row
				$sql = "INSERT INTO `".DBPREFIX."tree` ( `id` , `sort_id` , `parent_id` , `cdate` , `mdate` , `ip` ) 
				VALUES ('', '1', '$parent_id', '".date("Y-m-d H:i:s")."', '".date("Y-m-d H:i:s")."', '".$IP."')";
				$this->query($sql);
				$obj = $this->query("SELECT max(id) FROM ".DBPREFIX."tree"); 
				$lastid = $obj->fields['max(id)'];  
				$this->query("UPDATE ".DBPREFIX."tree SET sort_id=".$lastid." WHERE id=".$lastid); 

				foreach($data["row"] as $index=>$val) {
					if( !get_magic_quotes_gpc() ) {	$val = addslashes($val); }
					if($index!="IP" AND $index!="DATE_CREATE" AND $index!="id" AND $index!="ID"  AND $index!="cdate") {
						$sql = "INSERT INTO `".DBPREFIX."data` ( `id` , `parent_id` , `var` , `data` ) 
						VALUES ('', '$lastid', '$index', '$val')";
						$this->query($sql); 
					}
				}
			}

			if( $data["post_action"] == "CHANGEROW" OR $data["post_action"] == "UPDATEROW") {
				$sql = "UPDATE ".DBPREFIX."tree SET mdate='".date("Y-m-d H:i:s")."' WHERE id=".$data["row"]["id"];
				$this->query($sql);
				$sql = "DELETE FROM ".DBPREFIX."data WHERE parent_id=".$data["row"]["id"];
				$this->query($sql); 

				foreach($data["row"] as $index=>$val) {
					if( !get_magic_quotes_gpc() ) {	$val = addslashes($val); }
					if($index!="IP" AND $index!="DATE_CREATE" AND $index!="id" AND $index!="ID"  AND $index!="cdate")
						$sql = "INSERT INTO `".DBPREFIX."data` ( `id` , `parent_id` , `var` , `data` ) 
						VALUES ('', '".$data["row"]["id"]."', '$index', '$val')";
						$this->query($sql); 
				}				
					
			}
			
		}
		return $lastid;
	}	
	
	
	/**
	 * Delete the record
	 *
	 * @param string record identinicator (CDATE)
	 * @param string selecting parameter
	 * @return boolean
	 * @access public
	 */	
	function delete_data_row($arg_value, $arg_key="DATE_CREATE"){
		
		$r = false;
		$sql = "";
		$fetch = array();
		$index = "";
		$val = "";
		$id = 0;

		if($arg_key=="DATE_CREATE" or $arg_key=="cdate") {
		$sql = "SELECT id FROM ".DBPREFIX."tree WHERE cdate='".$arg_value."'";
		$fetch = $this->row_fetch_array($sql); 
		$id = $fetch["id"];
		} elseif($arg_key=="mdate") {
		$sql = "SELECT id FROM ".DBPREFIX."tree WHERE mdate='".$arg_value."'";
		$fetch = $this->row_fetch_array($sql); 
		$id = $fetch["id"];
		} elseif($arg_key=="id") {
			$id = $arg_value;
		} else { 		
		$sql = "SELECT parent_id FROM ".DBPREFIX."data WHERE var='".$arg_key."' AND `data`='".$arg_value."'";
		$fetch = $this->row_fetch_array($sql); 
		$id = $fetch["parent_id"];
		}
		
		$sql = "DELETE FROM ".DBPREFIX."data WHERE parent_id=".$id;
		$r=$this->query($sql); 		
		
		$sql = "DELETE FROM ".DBPREFIX."tree WHERE id=".$id;
		$r=$this->query($sql);
		return true;
	}

	/**
	 * Update the record
	 *
	 * @param string record identinicator (CDATE)
	 * @param array new record data
	 * @return boolean
	 * @access public
	 */	
	function change_data_row($date_create, $new_row){
		if(!$new_row) return false;
		$sql = "SELECT id FROM ".DBPREFIX."tree WHERE cdate='".$date_create."'";
		$fetch = $this->row_fetch_array($sql); 
		$id = $fetch["id"];		
		
				$sql = "UPDATE ".DBPREFIX."tree SET mdate='".date("Y-m-d H:i:s")."' WHERE id=".$id;
				$this->query($sql);
				$sql = "DELETE FROM ".DBPREFIX."data WHERE parent_id=".$id;
				$this->query($sql); 
				
				foreach($new_row as $index=>$val) {
					if( !get_magic_quotes_gpc() ) {	$val = addslashes($val); }
					if($index!="IP" AND $index!="DATE_CREATE" AND $index!="id"  AND $index!="cdate")
						$sql = "INSERT INTO `".DBPREFIX."data` ( `id` , `parent_id` , `var` , `data` ) 
						VALUES ('', '$id', '$index', '$val')";
						$this->query($sql); 
				}				
		return true;
	}		
	

}
?>